local mod = ...



-- Job: Cattle -------------------------------------------------------------------------------------
mod:registerAsset({
	DataType = "JOB",
	Id = "JOB_MONASTICLIFE_CATTLE",
	JobName = "JOB_MONASTICLIFE_CATTLE_NAME",
	JobDescription = "JOB_MONASTICLIFE_CATTLE_DESC",
	IsLockedByDefault = true,
	DefaultBehavior = "BEHAVIOR_GATHER",
	UseWorkplaceBehavior = true,
    ProductionDelay = 50, -- default 30
	AssetJobProgression = "DEFAULT_JOB_PROGRESSION",
	CharacterSetup = {
		RightHandObject = {
			DataType = "HAND_OBJECT",
			Model = "PREFAB_TOOL_SHEPHERD_STICK"
		},
        LeftHandObject = {
            Model = "PREFAB_TOOL_DAIRY_FARMER_BUCKET"
        },
        -- FemaleHatList = {
        --     "PREFAB_LIVESTOCKFARM_HAT",
        -- }, 
        -- MaleHatList = {
        --     "PREFAB_LIVESTOCKFARM_HAT",
        -- }, 
		WalkAnimation = "WALKING",
		IdleAnimation = "MILK",
	},
})

mod:overrideAsset({
    Id = "MONK",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_MONASTICLIFE_CATTLE",
    },
})

mod:overrideAsset({
    Id = "MONK_PRIOR",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_MONASTICLIFE_CATTLE",
    },
})



-- Building Function: Cattle --------------------------------------------------------------------------------------------
mod:registerAsset({
    DataType = "BUILDING_FUNCTION_LIVESTOCK_FARM",
    Id = "BUILDING_FUNCTION_MONASTICLIFE_CATTLE",
    Name = "BUILDING_FUNCTION_MONASTICLIFE_CATTLE_NAME",
    IsPrivate = true,
    WorkerCapacity = 1,
    RelatedJob = {
        Job = "JOB_MONASTICLIFE_CATTLE",
        Behavior = "BEHAVIOR_GATHER",
    },
    SpawningLivestock = "PREFAB_LIVESTOCK_COW",
    LivestockSpawningLocation = "Spawn.Cattle",
    SpawnLivestockButtonText = "BUTTON_MONASTICLIFE_SPAWN",
    RemoveLivestockButtonText = "BUTTON_MONASTICLIFE_DESPAWN",
    -- InputInventoryCapacity = {
	-- 	{ Resource = "MILK", Quantity = 1 },
	-- },
    ResourceListNeeded = {
		{ Resource = "MILK", Quantity = 1 },
	},
    ResourceProduced = {
		{ Resource = "MILK", Quantity = 2 },
		{ Resource = "RESOURCE_VELLUM", Quantity = 1 },
        { Resource = "RESOURCE_DUNG", Quantity = 1 },
	},
})



-- Building Part: Cattle -------------------------------------------------------------------------------------

-- Assets
mod:registerAssetId("models/Monastery1.fbx/Prefab/Cattle.Part.001", "PREFAB_MONASTICLIFE_CATTLE1" )
mod:registerAssetId("models/Monastery1.fbx/Prefab/Cattle.Part.001.Constr", "PREFAB_MONASTICLIFE_CATTLE1_CONSTR" )

-- Grounding
mod:registerPrefabComponent("models/Monastery1.fbx/Prefab/Cattle.Part.001/Bucket.001", {DataType = "COMP_GROUNDED"})
mod:registerPrefabComponent("models/Monastery1.fbx/Prefab/Cattle.Part.001/DungHeap.001", {DataType = "COMP_GROUNDED"})
mod:registerPrefabComponent("models/Monastery1.fbx/Prefab/Cattle.Part.001/Fence.001", {DataType = "COMP_GROUNDED"})
mod:registerPrefabComponent("models/Monastery1.fbx/Prefab/Cattle.Part.001/Fence.002", {DataType = "COMP_GROUNDED"})
mod:registerPrefabComponent("models/Monastery1.fbx/Prefab/Cattle.Part.001/Fence.003", {DataType = "COMP_GROUNDED"})
mod:registerPrefabComponent("models/Monastery1.fbx/Prefab/Cattle.Part.001/Fence.004", {DataType = "COMP_GROUNDED"})
mod:registerPrefabComponent("models/Monastery1.fbx/Prefab/Cattle.Part.001/Fence.005", {DataType = "COMP_GROUNDED"})
mod:registerPrefabComponent("models/Monastery1.fbx/Prefab/Cattle.Part.001/Fence.006", {DataType = "COMP_GROUNDED"})
mod:registerPrefabComponent("models/Monastery1.fbx/Prefab/Cattle.Part.001/Hay.001", {DataType = "COMP_GROUNDED"})
mod:registerPrefabComponent("models/Monastery1.fbx/Prefab/Cattle.Part.001/Hay.002", {DataType = "COMP_GROUNDED"})

-- Transparency
mod:registerAssetId("models/Monastery1.fbx/Materials/Material.Hay2", "MATERIAL_MONASTICLIFE_HAY2")
mod:overrideAsset({
    Id = "MATERIAL_MONASTICLIFE_HAY2",
    HasAlphaTest = true,
})

-- Building Part
local poly1 = { -- y, x     Facing x in Blender: y increases going left, x increases going down
	{ 3.1, 0.0  },
	{ 3.0, -5.0 },
	{ 2.8, -6.5 },
	{ 0.0, -7.0 },
	{ -2.0, -7.0 },
	{ -3.0, -6.0 },
	{ -3.1, -2.0 },
	{ -3.1, 3.5 },
	{ -3.9, 4.0 },
	{ -4.0, 6.0 },
	{ 0.0, 7.0 },
	{ 2.5, 6.5 },
	{ 3.0, 3.0 },
}

mod:registerAsset({
	DataType = "BUILDING_PART",
	Id = "BUILDING_PART_MONASTICLIFE_CATTLE1",
	Name = "BUILDING_PART_MONASTICLIFE_CATTLE_NAME",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "PREFAB_MONASTICLIFE_CATTLE1",
        MiniatureConfig = {
            ClipUnderGround = true,
            GroupHeight = 0,
            CameraPosition = { 0, 0, 0 },
            OrientationOffset = -90,
            CameraPitchOffset = 0.0
        },
	},
    ConstructionVisual = "PREFAB_MONASTICLIFE_CATTLE1_CONSTR",
	BuildingZone = {
        ZoneEntryList = {
            {
                Polygon = poly1,
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = false }
			},
            {
                Polygon = polygon.createRectangle( {4, 4}, {0.0, 0.0} ),
                Type = { DEFAULT = false, NAVIGABLE = true, GRASS_CLEAR = true }
			},
        },
	},
    Cost = {
        BuildRightTaxes = {
            { Resource = "GOLD_COINS", Quantity = 20 }
        },
        UpkeepCost = {
            { Resource = "GOLD_COINS", Quantity = 10 }
        },
        ResourceNeededList = {
            {
                { Resource = "WOOD", Quantity = 20 },
            },
            {
                { Resource = "STONE", Quantity = 20 },
            },
            {
                { Resource = "STONE", Quantity = 20 },
                { Resource = "TOOL", Quantity = 2 },
            },
            {
                { Resource = "PLANK", Quantity = 20 },
            },
        },
    },
	AssetBuildingFunction = "BUILDING_FUNCTION_MONASTICLIFE_CATTLE",
})


