local mod = ...


-- Prefab: Mushroom Tree -------------------------------------------------------------------------------------
-- Acts as a gatherable for mushroom foragers
mod:registerAssetId("models/Monastery1.fbx/Prefab/MushroomTree.Part", "PREFAB_MONASTICLIFE_MUSHROOMTREE" )



-- Resource container -------------------------------------------------------------------------------------
mod:registerPrefabComponent( "PREFAB_MONASTICLIFE_MUSHROOMTREE", {
    DataType = "COMP_RESOURCE_CONTAINER",
    ResourceData = "RESOURCE_MUSHROOM",
    IsReplenishable = true,
    ReplenishDurationInDays = 5,
    ReplenishQuantity = 1,
    ResourceValue = 1.0, -- how many are gathered with each pick
    AvailableQuantity = 1, -- maximum container stock
    Radius = 1.0, -- distance from container that the villager will stand to pick
    IsDestroyWhenEmpty = false,
})



-- Component: Mushroom Tree
local COMP_MONASTICLIFE_MUSHROOM_TREE = {
	TypeName = "COMP_MONASTICLIFE_MUSHROOM_TREE",
	ParentType = "COMPONENT",
	Properties = {
		-- { Name = "Timer", Type = "integer", Default = 2, Flags = { "SAVE_GAME" } },
    }
}

-- Register custom Class
mod:registerClass(COMP_MONASTICLIFE_MUSHROOM_TREE)

-- Attach class to prefab
mod:registerPrefabComponent("PREFAB_MONASTICLIFE_MUSHROOMTREE", { DataType = "COMP_MONASTICLIFE_MUSHROOM_TREE" } )



-- Component: Manager
local COMP_MONASTICLIFE_MUSHROOM_MANAGER = {
	TypeName = "COMP_MONASTICLIFE_MUSHROOM_MANAGER",
	ParentType = "COMPONENT",
	Properties = {
    }
}

-- onEnabled
function COMP_MONASTICLIFE_MUSHROOM_MANAGER:onEnabled()

    -- Weekly event
    local compMainGameLoop = self:getLevel():find("COMP_MAIN_GAME_LOOP")
    event.register(self, compMainGameLoop.ON_NEW_WEEK, function()

        -- Delete existing mushroom trees
        self:getLevel():getComponentManager("COMP_MONASTICLIFE_MUSHROOM_TREE"):getAllEnabledComponent():forEach(function(compMush)
            compMush:getOwner():destroy()
        end)

        -- Respawn mushroom trees around a portion of existing trees
        self:getLevel():getComponentManager("COMP_TREE"):getAllEnabledComponent():forEach(function(compTree)
            local chance = math.random()
            if chance < 0.05 then
                local objMush = self:getLevel():createObject("PREFAB_MONASTICLIFE_MUSHROOMTREE", compTree:getOwner():getGlobalPosition())
            end
        end)

    end)

end

-- Register custom Class
mod:registerClass(COMP_MONASTICLIFE_MUSHROOM_MANAGER)

-- Attach class to prefab
mod:registerPrefabComponent("PREFAB_MANAGER", { DataType = "COMP_MONASTICLIFE_MUSHROOM_MANAGER" } )



-- Job: Mushrooms -------------------------------------------------------------------------------------
mod:registerAsset({
	DataType = "JOB",
	Id = "JOB_MONASTICLIFE_MUSHROOM",
	JobName = "JOB_MONASTICLIFE_MUSHROOM_NAME",
	JobDescription = "JOB_MONASTICLIFE_MUSHROOM_DESC",
	IsLockedByDefault = true,
	DefaultBehavior = "BEHAVIOR_GATHER",
	UseWorkplaceBehavior = true,
    ProductionDelay = 10, -- default 30
	AssetJobProgression = "DEFAULT_JOB_PROGRESSION",
	CharacterSetup = {
		LeftHandObject = {
			DataType = "HAND_OBJECT",
			Model = "PREFAB_TOOL_FARMER_SEED_BAG",
            WorkAttachName = "SeedBag_Anchor",
            WalkAttachName = "SeedBag_Anchor",
		},
        WalkAnimation = "WALKING",
		IdleAnimation = "GATHER",
	},
})

mod:overrideAsset({
    Id = "MONK",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_MONASTICLIFE_MUSHROOM",
    },
})

mod:overrideAsset({
    Id = "MONK_PRIOR",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_MONASTICLIFE_MUSHROOM",
    },
})



-- Building Function: Mushrooms --------------------------------------------------------------------------------------------
mod:registerAsset({
    DataType = "BUILDING_FUNCTION_WORKPLACE",
    Id = "BUILDING_FUNCTION_MONASTICLIFE_MUSHROOM",
    Name = "BUILDING_FUNCTION_MONASTICLIFE_MUSHROOM_NAME",
    IsPrivate = true,
    WorkerCapacity = 2,
    RelatedJob = {
        Job = "JOB_MONASTICLIFE_MUSHROOM",
        Behavior = "BEHAVIOR_GATHER",
    },
    InputInventoryCapacity = {
    },
    ResourceListNeeded = {
        { Resource = "RESOURCE_MUSHROOM", Quantity = 2 },
    },
    ResourceProduced = {
        { Resource = "RESOURCE_MUSHROOM", Quantity = 2 },
    },
})



-- Building Part: Mushroom Hut -------------------------------------------------------------------------------------

-- Assets
mod:registerAssetId("models/Monastery1.fbx/Prefab/Mushroom.Part.001", "PREFAB_MONASTICLIFE_MUSHROOM1" )
mod:registerAssetId("models/Monastery1.fbx/Prefab/Mushroom.Part.001.Constr", "PREFAB_MONASTICLIFE_MUSHROOM1_CONSTR" )
mod:registerAssetId("models/Monastery1.fbx/Prefab/Mushroom.Part.001/Door.010", "PREFAB_MONASTICLIFE_MUSHROOM1_DOOR" )

-- Grounded
mod:registerPrefabComponent("PREFAB_MONASTICLIFE_MUSHROOM1_DOOR", {DataType = "COMP_GROUNDED"})

-- Part
mod:registerAsset({
	DataType = "BUILDING_PART",
	Id = "BUILDING_PART_MONASTICLIFE_MUSHROOM1",
	Name = "BUILDING_PART_MONASTICLIFE_MUSHROOM_NAME",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
        CoreObjectPrefab = "PREFAB_MONASTICLIFE_MUSHROOM1",
        MiniatureConfig = {
            ClipUnderGround = true,
            GroupHeight = 0,
            CameraPosition = { 0, 0, 0 },
            OrientationOffset = 0,
            CameraPitchOffset = 0.0
        },
	},
    ConstructionVisual = "PREFAB_MONASTICLIFE_MUSHROOM1_CONSTR",
    AssetBuildingFunction = "BUILDING_FUNCTION_MONASTICLIFE_MUSHROOM",
    BuildingZone = {
        ZoneEntryList = {
            {
                Polygon = polygon.createCircle( 3.5, {0.0, 0.0}, 32 ),
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true },
			},
        },
    },
    Cost = {
        BuildRightTaxes = {
            { Resource = "GOLD_COINS", Quantity = 10 },
        },
        UpkeepCost = {
            { Resource = "GOLD_COINS", Quantity = 5 },
        },
        ResourceNeededList = {
            {
                { Resource = "STONE", Quantity = 10 },
                { Resource = "WOOD", Quantity = 10 },
            },
            {
                { Resource = "POLISHED_STONE", Quantity = 10 },
                { Resource = "TOOL", Quantity = 2 },
            },
        },
    },
})