local mod = ...



-- Job: Storehouse -------------------------------------------------------------------------------------
mod:registerAsset({
	DataType = "JOB",
	Id = "JOB_MONASTICLIFE_STOREHOUSE",
	JobName = "JOB_MONASTICLIFE_STOREHOUSE_NAME",
	JobDescription = "JOB_MONASTICLIFE_STOREHOUSE_DESC",
	IsLockedByDefault = true,
	DefaultBehavior = "BEHAVIOR_TRANSPORT",
	UseWorkplaceBehavior = true,
    ProductionDelay = 1, -- default 30
	AssetJobProgression = "DEFAULT_JOB_PROGRESSION",
	CharacterSetup = {
        -- FemaleHatList = {
        --     "PREFAB_LIVESTOCKFARM_HAT",
        -- }, 
        -- MaleHatList = {
        --     "PREFAB_LIVESTOCKFARM_HAT",
        -- }, 
		WalkAnimation = "WALKING",
		IdleAnimation = "IDLE",
	},
})

mod:overrideAsset({
    Id = "MONK",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_MONASTICLIFE_STOREHOUSE",
    },
})

mod:overrideAsset({
    Id = "MONK_PRIOR",
    CompatibleJobList = {
        Action = "APPEND",
        "JOB_MONASTICLIFE_STOREHOUSE",
    },
})



-- Building Function: Storehouse --------------------------------------------------------------------------------------------
mod:registerAsset({
    DataType = "BUILDING_FUNCTION_WAREHOUSE",
    Id = "BUILDING_FUNCTION_MONASTICLIFE_STOREHOUSE",
    Name = "BUILDING_FUNCTION_MONASTICLIFE_STOREHOUSE_NAME",
    IsPrivate = true,
    WorkerCapacity = 2,
    RelatedJob = {
        Job = "JOB_MONASTICLIFE_STOREHOUSE",
        Behavior = "BEHAVIOR_TRANSPORT",
    },
    SocketCount = 1,
    SocketCapacity = 100,
    AllowedResourceTypeList = {
        {
            ResourceType = "RESOURCE_TYPE_MONASTERY_FOOD",
            ShowInDescription = true,
        },
        {
            ResourceType = "GOODS",
            ShowInDescription = true,
        },
        {
            ResourceType = "LUXURY",
            ShowInDescription = true,
        },
        {
            ResourceType = "LUXURY_FOOD",
            ShowInDescription = true,
        },
    },
    DefaultResourceVisual = "PREFAB_RESOURCE_VISUAL_CRATE",
})



-- Building Part: Storehouse -------------------------------------------------------------------------------------
mod:registerAssetId("models/Monastery1.fbx/Prefab/Storehouse.Part.001", "PREFAB_MONASTICLIFE_STOREHOUSE1" )
mod:registerAssetId("models/Monastery1.fbx/Prefab/Storehouse.Part.001/Socket_0", "PREFAB_MONASTICLIFE_STOREHOUSE1_SOCKET0" )
mod:registerAssetId("models/Monastery1.fbx/Prefab/Inventory", "PREFAB_MONASTICLIFE_INVENTORY" )

mod:registerPrefabComponent( "PREFAB_MONASTICLIFE_INVENTORY", {
    DataType = "COMP_INVENTORY",
    Inventory = {},
})

mod:registerPrefabComponent( "PREFAB_MONASTICLIFE_STOREHOUSE1_SOCKET0", {
    DataType = "COMP_RESOURCE_DEPOT",
    Capacity = 100,
    OutgoingReservedResources = {
    },
    IncomingReservedResources = {
    },
})

-- mod:registerPrefabComponent( "PREFAB_MONASTICLIFE_STOREHOUSE1_SOCKET0", {
--     DataType = "COMP_RESOURCE_STOCKPILE",
--     ResourceLayoutTypeList = {
--         "CRATES",
--     },
--     LocatorPrefix = "Socket",
--     DefaultResourceVisual = "PREFAB_RESOURCE_VISUAL_CRATE",
--     ResourceStockpileVisualMode = "WAREHOUSE_SPAWN",
-- })

mod:registerAsset({
	DataType = "BUILDING_PART",
	Id = "BUILDING_PART_MONASTICLIFE_STOREHOUSE1",
	Name = "BUILDING_PART_MONASTICLIFE_STOREHOUSE_NAME",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "PREFAB_MONASTICLIFE_STOREHOUSE1",
        MiniatureConfig = {
            ClipUnderGround = true,
            GroupHeight = 0,
            CameraPosition = { 0, 0, 0 },
            OrientationOffset = 0,
            CameraPitchOffset = 0.0
        },
	},
	-- BuildingZone = {
    --     ZoneEntryList = {
    --         {
    --             Polygon = polygon.createRectangle( {6, 3}, {0.0, 0.0} ),
    --             Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true }
	-- 		},
	-- 	},
	-- },
	-- Cost = {
	-- 	ResourceNeededList = {
    --         {
    --             { Resource = "TOOL", Quantity = 1 },
    --         },
	-- 	}
	-- },
	AssetBuildingFunction = "BUILDING_FUNCTION_MONASTICLIFE_STOREHOUSE",
})

-- Component
local COMP_MONASTICLIFE_STOREHOUSE_SOCKET = {
	TypeName = "COMP_MONASTICLIFE_STOREHOUSE_SOCKET",
	ParentType = "COMPONENT",
	Properties = {
    }
}


-- onEnabled
function COMP_MONASTICLIFE_STOREHOUSE_SOCKET:onEnabled()

    local obj = self:getOwner()

    local objInput = self:getLevel():createObject("PREFAB_MONASTICLIFE_INVENTORY")
    objInput:setParent(obj)

    local objOutput = self:getLevel():createObject("PREFAB_MONASTICLIFE_INVENTORY")
    objOutput:setParent(obj)

    local compInput = objInput:getOrCreateComponent("COMP_INVENTORY")
    local compOutput = objOutput:getOrCreateComponent("COMP_INVENTORY")

    mod:log("Set inventory objects on Socket.")

end


-- Register custom Class
mod:registerClass(COMP_MONASTICLIFE_STOREHOUSE_SOCKET)

-- Attach class to prefab
mod:registerPrefabComponent("PREFAB_MONASTICLIFE_STOREHOUSE1_SOCKET0", { DataType = "COMP_MONASTICLIFE_STOREHOUSE_SOCKET" } )



-- Building Part: Cave -------------------------------------------------------------------------------------
mod:registerAssetId("models/Monastery1.fbx/Prefab/Hut.Part.004", "PREFAB_MONASTICLIFE_HUT4" )
mod:configurePrefabFlagList("PREFAB_MONASTICLIFE_HUT4", { "PLATFORM" })

mod:registerAsset({
	DataType = "BUILDING_PART",
	Id = "BUILDING_PART_MONASTICLIFE_HUT4",
	Name = "BUILDING_PART_MONASTICLIFE_HALL_NAME",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = "PREFAB_MONASTICLIFE_HUT4",
        MiniatureConfig = {
            ClipUnderGround = true,
            GroupHeight = 0,
            CameraPosition = { 0, 0, 0 },
            OrientationOffset = 0,
            CameraPitchOffset = 0.0
        },
	},
    AssetBuildingFunction = "BUILDING_FUNCTION_MONASTICLIFE_DORM",
    Capacity = 4,
	-- BuildingZone = {
    --     ZoneEntryList = {
    --         {
    --             Polygon = polygon.createRectangle( {6, 3}, {0.0, 0.0} ),
    --             Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true }
	-- 		},
	-- 	},
	-- },
	-- Cost = {
	-- 	ResourceNeededList = {
    --         {
    --             { Resource = "TOOL", Quantity = 1 },
    --         },
	-- 	}
	-- },
})

