local mod = ...



-- Assets
mod:registerAssetId("models/Upgrade1.fbx/Prefab/VegetableFarm.Tools.Part", "PREFAB_MONASTICLIFE_UPGRADE_VEGETABLE_FARM_TOOLS" )



-- Building Function: Vegetable Plot -------------------------------------------------------------------------------------
mod:registerAsset({
    DataType = "BUILDING_FUNCTION_WORKPLACE",
    Id = "BUILDING_FUNCTION_VEGETABLEFARM_PLOT_UPGRADE",
    Name = "BUILDING_FUNCTION_VEGETABLEFARM_PLOT_NAME",
    WorkerCapacity = 1,
    RelatedJob = {
        Job = "JOB_VEGETABLEFARM_PLOT",
        Behavior = "BEHAVIOR_WORK",
    },
    InputInventoryCapacity = {
		{ Resource = "WATER", Quantity = 10 },
		{ Resource = "RESOURCE_COMPOST", Quantity = 10 }
	},
    ResourceListNeeded = {
		{ Resource = "WATER", Quantity = 1 },
		{ Resource = "RESOURCE_COMPOST", Quantity = 1 },
	},
	ResourceProduced = {
        { Resource = "RESOURCE_VEGETABLE", Quantity = 2 },
    },
})



-- Building Function: Vegetable shed -------------------------------------------------------------------------------------
mod:registerAsset({
    DataType = "BUILDING_FUNCTION_WORKPLACE",
    Id = "BUILDING_FUNCTION_VEGETABLEFARM_SHED_UPGRADE",
    Name = "BUILDING_FUNCTION_VEGETABLEFARM_SHED_NAME",
    WorkerCapacity = 1,
    RelatedJob = {
        Job = "JOB_VEGETABLEFARM_SHED",
        Behavior = "BEHAVIOR_WORK",
    },
    InputInventoryCapacity = {
		{ Resource = "WATER", Quantity = 20 },
		{ Resource = "RESOURCE_COMPOST", Quantity = 20 },
		{ Resource = "RESOURCE_VEGETABLE_RAW", Quantity = 20 },
	},
    ResourceListNeeded = {
		{ Resource = "WATER", Quantity = 1 },
		{ Resource = "RESOURCE_COMPOST", Quantity = 1 },
		{ Resource = "RESOURCE_VEGETABLE_RAW", Quantity = 5 },
	},
	ResourceProduced = {{ Resource = "RESOURCE_VEGETABLE", Quantity = 16 }}
})



-- Upgrade: Vegetable Farm Tools -------------------------------------------------------------------------------------

-- Upgrade Component to attach to upgraded Building Part
local COMP_MONASTICLIFE_UPGRADE_MONUMENT_PART = {
    TypeName = "COMP_MONASTICLIFE_UPGRADE_MONUMENT_PART",
    ParentType = "COMPONENT",
    Properties = {
        {
            Name = "InitFlag",
            Type = "boolean",
            Default = false,
            Flags = { "SAVE_GAME" }
        },
    }
}

-- OnEnabled
function COMP_MONASTICLIFE_UPGRADE_MONUMENT_PART:onEnabled()

    if not self.InitFlag then

        local objPart = self:getOwner()
        local compWork = objPart:getComponent("COMP_WORKPLACE")

        -- Listen for production
        event.register(self, compWork.ON_WORKPLACE_PRODUCED, function(resourceProduced)

            local qtyInvAdd = resourceProduced.Collection[1].Quantity

            mod:log("Vegetable farm produced.")

            local objDepot = objPart:getComponent("COMP_RESOURCE_DEPOT")
            if objDepot then

                local compInvOut = objDepot.OutputInventory
                if compInvOut then
                    local qtyInvOut = compInvOut.Inventory.Collection[1].Quantity
                    if qtyInvOut then
                        compInvOut.Inventory.Collection[1].Quantity = qtyInvOut + qtyInvAdd
                    end

                    mod:log("Added " .. qtyInvAdd .. " to existing inventory of " .. qtyInvOut)
                    
                    compWork:update()

                end

            end

            -- for i = 1, #listResource.Collection do
            --     mod:log("Resource " .. tostring(listResource.Collection[i].Resource) .. ", Quantity " .. tostring(listResource.Collection[i].Quantity))
            -- end

        end)

        mod:log("Listener placed on vegetable farm.")

        self.InitFlag = true

    end

end

-- Register Component
mod:registerClass(COMP_MONASTICLIFE_UPGRADE_MONUMENT_PART)



-- Upgrade Component to attach to Upgrade
local COMP_MONASTICLIFE_UPGRADE_VEGETABLE_FARM = {
    TypeName = "COMP_MONASTICLIFE_UPGRADE_VEGETABLE_FARM",
    ParentType = "COMPONENT",
    Properties = {
    }
}

-- OnEnabled
function COMP_MONASTICLIFE_UPGRADE_VEGETABLE_FARM:onEnabled()

    local gameObject = self:getOwner()

    -- Find the nearest non-upgraded building part
    local compBuilding = gameObject:findFirstObjectWithComponentUp("COMP_BUILDING")
    if compBuilding then

        -- Get list of building parts
        local listPart = compBuilding:getBuildingPartList()
        if listPart then

            -- There is at least one building part
            if #listPart > 0 then

                local maxDist = 1000
                local objPart = nil
                local posUpgrade = gameObject:getGlobalPosition()

                -- Loop through parts
                for i = 1, #listPart do

                    local objCurrent = listPart[i]:getOwner()

                    -- The part must not already be upgraded
                    local compUpgrade = objCurrent:getComponent("COMP_MONASTICLIFE_UPGRADE_MONUMENT_PART")
                    if not compUpgrade then

                        -- The part must be a workplace
                        local compWork = objCurrent:getComponent("COMP_WORKPLACE")
                        if compWork then

                            -- Find out if the part is closer than any other parts seen so far, if so store it as the final part
                            local posPart = objCurrent:getGlobalPosition()
                            local dist = math.sqrt( (posUpgrade[1] - posPart[1])^2 + (posUpgrade[3] - posPart[3])^2 )

                            if dist < maxDist then
                                objPart = objCurrent
                                maxDist = dist
                            end

                        end

                    end

                end

                -- If a matching part was found, attach the upgrade
                if objPart then

                    local compUpgrade = objPart:addComponent("COMP_MONASTICLIFE_UPGRADE_MONUMENT_PART")

                end

            end
        end

    end

    -- Disable this component
    gameObject:removeComponent(self)

end

-- Register Component
mod:registerClass(COMP_MONASTICLIFE_UPGRADE_VEGETABLE_FARM)

-- Attach Component
mod:registerPrefabComponent("PREFAB_MONASTICLIFE_UPGRADE_VEGETABLE_FARM_TOOLS", { DataType = "COMP_MONASTICLIFE_UPGRADE_VEGETABLE_FARM"})



-- Assemblage
mod:registerAsset({
	DataType = "BUILDING_PART",
	Id = "BUILDING_PART_MONASTICLIFE_UPGRADE_VEGETABLE_FARM_TOOLS_ASSEMBLAGE",
	Name = "BUILDING_PART_MONASTICLIFE_UPGRADE_TOOLS_NAME",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_ASSEMBLAGE",
		CoreRandomBuildingPartList = { "PREFAB_MONASTICLIFE_UPGRADE_VEGETABLE_FARM_TOOLS" },
		MandatoryBuildingPartList = {
			{ BuildingPart = "TOOLS_BARREL_FARM_PART" }, -- from Fantasy Decorations
			{ BuildingPart = "TOOLS_RACK_FARM_PART" }, -- from Fantasy Decorations
		}
	},
})

-- Random
mod:registerAsset({
    DataType = "BUILDING_PART",
    Id = "BUILDING_PART_MONASTICLIFE_UPGRADE_VEGETABLE_FARM_TOOLS",
    Name = "BUILDING_PART_MONASTICLIFE_UPGRADE_TOOLS_NAME",
    Description = "BUILDING_PART_MONASTICLIFE_UPGRADE_TOOLS_MONUMENT_DESC",
    Category = "UPGRADE",
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_RANDOM_PART",
        RandomPartList = {
            "BUILDING_PART_MONASTICLIFE_UPGRADE_VEGETABLE_FARM_TOOLS_ASSEMBLAGE",
        },
        MiniatureConfig = {
            ClipUnderGround = true,
            GroupHeight = 0,
            CameraPosition = { 0, 0, 0 },
            OrientationOffset = 0,
            CameraPitchOffset = 0.0
        },
    },
    HasMaximumInstancePerBuilding = true,
    MaximumInstancePerBuilding = 1,
    BuildingZone = {
        ZoneEntryList = {
            {
                Polygon = polygon.createRectangle( {4.0, 2.25}, {0.0, 0.0} ),
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true },
            },
        },
    },
    -- Cost = {
    --     BuildRightTaxes = {
    --         { Resource = "RESOURCE_LEARNING_PLANTS", Quantity = 50 },
    --         { Resource = "GOLD_COINS", Quantity = 50 },
    --     },
    --     UpkeepCost = {
    --         { Resource = "GOLD_COINS", Quantity = 25 }
    --     },
    --     ResourceNeededList = {
    --         {
    --             { Resource = "TOOL", Quantity = 20 },
    --             { Resource = "WOOD", Quantity = 4 },
    --         },
    --     },
    -- },
})



-- Upgrade: Additional Field -------------------------------------------------------------------------------------

-- Assemblage
mod:registerAsset({
	DataType = "BUILDING_PART",
	Id = "BUILDING_PART_MONASTICLIFE_UPGRADE_VEGETABLE_FARM_WORKERS_ASSEMBLAGE",
	Name = "BUILDING_PART_MONASTICLIFE_UPGRADE_WORKERS_NAME",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_ASSEMBLAGE",
		CoreRandomBuildingPartList = { "PREFAB_MONASTICLIFE_UPGRADE_HOP_FARM_WORKERS" },
		MandatoryBuildingPartList = {
			{ BuildingPart = "SACK_CLOSED_PART" }, -- from Fantasy Decorations
			{ BuildingPart = "CRATE1_PART" }, -- from Fantasy Decorations
			{ BuildingPart = "BARREL1_PART" }, -- from Fantasy Decorations
		}
	},
})

-- Random
mod:registerAsset({
    DataType = "BUILDING_PART",
    Id = "BUILDING_PART_MONASTICLIFE_UPGRADE_VEGETABLE_FARM_WORKERS",
    Name = "BUILDING_PART_MONASTICLIFE_UPGRADE_WORKERS_NAME",
    Category = "UPGRADE",
    AssetBuildingFunction = "BUILDING_FUNCTION_ASSIGNABLE_VEGETABLEFARM",
    ConstructorData = {
        DataType = "BUILDING_CONSTRUCTOR_RANDOM_PART",
        RandomPartList = {
            "BUILDING_PART_MONASTICLIFE_UPGRADE_VEGETABLE_FARM_WORKERS_ASSEMBLAGE",
        },
        MiniatureConfig = {
            ClipUnderGround = true,
            GroupHeight = 0,
            CameraPosition = { 0, 0, 0 },
            OrientationOffset = 0,
            CameraPitchOffset = 0.0
        },
    },
    HasMaximumInstancePerBuilding = true,
    MaximumInstancePerBuilding = 1,
    BuildingZone = {
        ZoneEntryList = {
            {
                Polygon = polygon.createCircle( 1.0, {0.0, 0.0}, 16 ),
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true },
            },
        },
    },
    Cost = {
        BuildRightTaxes = {
                { Resource = "RESOURCE_LEARNING_PLANTS", Quantity = 25 },
                { Resource = "GOLD_COINS", Quantity = 10 },
        },
        UpkeepCost = {
            { Resource = "GOLD_COINS", Quantity = 15 }
        },
        ResourceNeededList = {
            {
                { Resource = "TOOL", Quantity = 3 },
            },
        },
    },
})



-- Add to Building
mod:overrideAsset({
    Id = "BUILDING_VEGETABLEFARM",
    BuildingPartSetList = {
        Action = "APPEND",
        {
            Name = "BUILDING_PART_SETLIST_MONASTICLIFE_UPGRADE",
            BuildingPartList = {
                "BUILDING_PART_MONASTICLIFE_UPGRADE_VEGETABLE_FARM_TOOLS",
                "BUILDING_PART_MONASTICLIFE_UPGRADE_VEGETABLE_FARM_WORKERS",
            },
        },
    },
})
